"""
Blask

Copyright (C) 2018  https://github.com/zerasul/blask

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""
from markdown import Markdown
from os import path, listdir
from Blask.errors import PageNotExistError
from datetime import datetime
from hashlib import sha256
from functools import lru_cache


class BlogRenderer:
    """
    Class BlogRenderer: This class provides the feature for render posts from
    Markdown to HTML and search features.
    :Author: Zerasul <suarez.garcia.victor@gmail.com>
    Date: 2018-05-05
    Version: 0.1.0
    """
    postdir = None
    """
    Posts Directory
    """
    cache = {}
    """
    Post Cache; improves the post loading.
    """
    def __init__(self, postdir):
        """
        This is the constructor of the blog renderer.
        :param postdir: Posts Directory. See Settings py for more information.
        """
        self.postdir = postdir

    def renderfile(self, filename):
        """
            Render a markdown and returns the blogEntry.
            Note: This method uses a cache based on a SHA-256 hash of the
            content.
        :param filename: Number of the file without extension.
        :return: BlogEntry.
        :raises PageNotExistError Raise this error if file does not exists.
        """
        filepath = path.join(self.postdir, filename + ".md")
        if not path.exists(filepath):
            raise PageNotExistError("{} does not exists in {} directory".format(filename, self.postdir))
        with open(filepath, 'r') as content_file:
            content = content_file.read()
            # Check cache
            content_hash = sha256(content.encode())
            if content_hash not in self.cache:
                entry = self.rendertext(filename, content)
                self.cache[content_hash] = entry

        return self.cache[content_hash]

    def rendertext(self, filename, text):
        """
         Render a Markdown Text and returns the BlogEntry.
        :param filename: filename or title of the post.
        :param text: Text write in Markdown.
        :return: BlogEntry.
        """
        md = Markdown(extensions=['meta', 'markdown.extensions.codehilite'])
        entry = BlogEntry(filename, md, text)
        return entry

    def list_posts(self, tags=[], exclusions=["index.md", "404.md"], search="",
                   category="", author=""):
        """
        Search a list of Posts returning a list of BlogEntry.
        :param tags: list of tags for searching.
        :param exclusions: list of name of posts with exclusions.
        :param search: string with the content what we want of search.
        :param category: list of category of the entry.
        :param author: name of the author of the post
        :return: List of BlogEntry.
        """
        files = list(filter(lambda l: l.endswith('.md') and l not in
                            exclusions, listdir(self.postdir)))
        mapfilter = list(map(lambda l: path.splitext(l)[0], files))
        entries = list(map(lambda l: self.renderfile(l), mapfilter))
        if tags:
            for tag in tags:
                entries = list(filter(lambda l: tag in l.tags, entries))
        if category:
            entries = list(filter(lambda c: c.category == category, entries))
        if author:
            entries = list(filter(lambda a: a.author == author, entries))
        if search:
            entries = list(filter(lambda l: search in l.content, entries))

        return entries

    def generatetagpage(self, postlist):
        """
        Get a HTML with links of the entries.
        :param postlist: List with BlogEntry.
        :return: String with the HTML list.
        """
        content = '<ul>'
        for post in postlist:
            entrycontent = "<li><a href='/{}'>{}</a></li>".format(post.name,
                                                                  post.name)
            content += entrycontent
        content += "</ul>"
        return content


class BlogEntry:
    """"
    This class has the information about the Blog Posts.
    Author: Zerasul
    Version: 0.0.1.
    """
    content = None
    """Content of the post."""
    date = None
    """ Date of post creation"""
    tags = []
    """List of tags of the blog entry."""
    author = None
    """Author of the post"""
    category = None
    """category of the post"""
    template = None
    """Name of the template file"""
    name = None
    """ Name of the post"""

    def __init__(self, name, md, content):
        """
        Default constructor
        :param name: name of the post
        :param md: Markdown information
        :param content: String with the Content in HTML.
        """
        self.content = md.convert(content)
        self.name = name
        meta = md.Meta
        if meta:
            self.date = datetime.strptime(meta['date'][0], '%Y-%m-%d')
            self.tags = meta['tags'][0].split(',')
            if 'template' in meta.keys():
                self.template = meta['template'][0]
            if 'category' in meta.keys():
                self.category = meta['category'][0]
            if 'author' in meta.keys():
                self.author = meta['author'][0]

    def __str__(self):
        string = "['content': {}, 'name': {}, ".format(self.content, self.name)
        string += "'date': {}, 'tags':[{}], ".format(self.date, self.tags)
        string += "'author': {}, 'category': {}, ".format(self.author,
                                                          self.category)
        string += "'template': {}]".format(self.template)

        return string



"""
Blask

Copyright (C) 2018  https://github.com/zerasul/blask

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""
import click

from Blask import BlaskApp, blasksettings
from os import mkdir, path, getcwd


class CLIController:

    default_template_file = """
<html>
    <head>
        <title>{{title}}</title>

        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <title>Hello Bulma!</title>
        <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/bulma/0.7.1/css/bulma.min.css">
        <script defer src="https://use.fontawesome.com/releases/v5.3.1/js/all.js"></script>
        <style>
            h1 {
                font-size: 2em;
            }
        </style>
    </head>
    <body>
        <section class="section">
            <div class="container">
                {{content|safe}}
            </div>
        </section>
    </body>
</html>
"""

    default_index = """# Welcome to Blask!
This is your `index.md` file, located at `posts` directory. Edit this file to add your own index. You can use Markdown to write it!

Blask is currently under development, and we have a lot of things to do.

Wanna help us?

Check [this project at GitHub](https://github.com/zerasul/blask)
"""

    settings = """ 
    # Minimal conf for Blask
    FLASK_APP=main.py                

    # Name of the Template Folder.
    templateDir="templates"

    # Name of the post Folder
    postDir="posts"

    # default name of the template file.
    defaultLayout='template.html'

    # Default name of the static Folder
    staticDir='static'

    # Title of the blog
    title='Blask | A Simple Blog Engine Based on Flask' 
    """

    not_found = "# 404\n Page not found"

    def createdefaultindexfile(self, filepath):
        with open(filepath, 'w') as indexfile:
            indexfile.write(self.default_index)

    def createdefaulttemplatefile(self, filepath):
        with open(filepath, 'w') as templatefile:
            templatefile.write(self.default_template_file)

    def createdir(self, dirpath):
            mkdir(dirpath)
            return True

    def createsettingsfile(self):
        with open(path.join(getcwd(), '.env'), 'w') as settingsFile:
            settingsFile.write(self.settings)

    def createnotfoundpage(self, filepath):
        with open(path.join(filepath, '404.md'), 'w') as page:
            page.write(self.not_found)


blask = BlaskApp()
isdebug = False
cliController = CLIController()


version = '0.1.1.1'


@click.group()
def blaskcli():
    click.echo('Blask (C) version %s' % version)


@blaskcli.command(help='Run the instance of blask')
@click.option('--debug', default=False, help="Init with de debug flag")
@click.option('--port', default=5000, help='Port where the server is listening')
def run(debug, port):
    blask.run(debug=debug, port=port)


@blaskcli.command(help='Initialize a new Blask Project')
def init():
    click.echo('Initializing new Blask Project')
    click.echo('Using default Settings')
    postdir = path.basename(path.dirname(str(blasksettings.DEFAULT_SETTINGS['postDir']+'/')))
    templatedir = path.basename(path.dirname(str(blasksettings.DEFAULT_SETTINGS['templateDir']+'/')))
    try:
        cliController.createdir(postdir)
        cliController.createdefaultindexfile(path.join(postdir, 'index.md'))
        cliController.createdir(templatedir)
        cliController.createdefaulttemplatefile(path.join(templatedir, 'template.html'))
        cliController.createsettingsfile() # creates a sample settings file
        cliController.createnotfoundpage(postdir) # creates a 404 page
        click.echo('Created new Blask project on %s' % getcwd())
        click.echo('Now you can execute: blaskcli run')
    except FileExistsError as e:
        print(str(e))
        click.echo("There is an existing Blask Project")


if __name__ == '__main__':
    blaskcli()



from setuptools import setup, find_packages
from pathlib import Path

info_file = Path(__file__).resolve().parents[0] / 'README.md'
with info_file.open() as f:
    long_desc = f.read()

setup(
    name='Blask',
    version='0.1.1.1',
    packages=find_packages(exclude=['tests']),
    url='https://getblask.com',
    license='GPL 3.0',
    author='zerasul',
    author_email='zerasul@gmail.com',
    description='A simple Blog engine using Flask and Markdown.',
    classifiers=[
       'Development Status :: 4 - Beta',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: News/Diary',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    entry_points='''
        [console_scripts]
        blaskcli=Blask.blaskcli:blaskcli
    ''',
    long_description=long_desc,
    long_description_content_type='text/markdown',
    python_requires='>=3',
    install_requires=[
          'flask',
          'markdown',
          'Pygments',
          'click'
    ],
    test_requires=[
        'pytest',
        'pytest-cov',
        'pylint'
    ]
)



"""
Blask

Copyright (C) 2018  https://github.com/zerasul/blask

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

import os
from pathlib import Path
from sys import path
from importlib import import_module


BASE_DIR = Path('.').resolve()

DEFAULT_SETTINGS = {
    'templateDir': str(BASE_DIR / 'templates'),
    'postDir': str(BASE_DIR / 'posts'),
    'defaultLayout': str('template.html'),
    'staticDir': str(BASE_DIR / 'static'),
    'title': 'Blask | A Simple Blog Engine Based on Flask'
}


class BlaskSettings(object):
    """
    Blask configuration helper class
    """

    def __init__(self, *args, **kwargs):
        # Check environment variable for settings module
        if 'BLASK_SETTINGS' in os.environ:
            #add current Dir to Path
            path.append(os.getcwd())
            # Load settings from the module in environment variable
            settings_mod = import_module(os.environ['BLASK_SETTINGS'], os.environ['BLASK_SETTINGS'])

            self.settings = {}
            for key in DEFAULT_SETTINGS.keys():
                value = getattr(settings_mod, key, DEFAULT_SETTINGS[key])
                self.settings[key] = value
        else:
            # Copy default settings
            self.settings = DEFAULT_SETTINGS.copy()

        # Keyword arguments always override default and environment settings
        for kw in kwargs.keys():
            if kw in DEFAULT_SETTINGS:
                self.settings[kw] = kwargs[kw]

    def __getitem__(self, key):
        """
        Dictionary like settings access
        """
        if key in self.settings:
            return self.settings[key]
        raise KeyError('There is no blask setting called %s' % key)



"""
Blask

Copyright (C) 2018  https://github.com/zerasul/blask

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from flask import Flask, render_template, request
from Blask.blasksettings import BlaskSettings
from Blask.blogrenderer import BlogRenderer
from Blask.errors import PageNotExistError


class BlaskApp:
    """
    Blask Application Main Class
    :Author: Zerasul <suarez.garcia.victor@gmail.com>
    date: 2018-05-05
    """
    app = None
    blogrenderer = None

    def __init__(self, **kwargs):
        self.settings = BlaskSettings(**kwargs)
        self.blogrenderer = BlogRenderer(self.settings['postDir'])
        self.app = Flask(__name__,
                         template_folder=self.settings['templateDir'],
                         static_folder=self.settings['staticDir'])
        self.app.add_url_rule('/', endpoint='index', view_func=self._index,
                              methods=['GET'])
        self.app.add_url_rule('/<filename>', view_func=self._getpage,
                              methods=['GET'])
        self.app.add_url_rule('/tag/<tag>', view_func=self._gettag,
                              methods=['GET'])
        self.app.add_url_rule('/search', view_func=self.searchpages,
                              methods=['POST'])
        self.app.add_url_rule('/category/<category>',
                              view_func=self._getcategory, methods=['GET'])
        self.app.add_url_rule('/author/<author>', view_func=self._getauthor,
                              methods=['GET'])

    def _index(self):
        """
        Render the Index page
        :return: rendered Index Page
        """
        entry = self.blogrenderer.renderfile("index")
        template = entry.template
        if template is None:
            template = self.settings['defaultLayout']
        return render_template(template, title=self.settings['title'],
                               content=entry.content)

    def _getpage(self, filename):
        """
        Render a blog post
        :param filename: Name of the Blog Post.
        :return: rendered Blog post or 404 page.
        """
        try:
            entry = self.blogrenderer.renderfile(filename)
        except PageNotExistError:
            entry = self.blogrenderer.renderfile("404")
        content = entry.content
        date = entry.date
        template = entry.template
        tags = entry.tags
        category = entry.category
        author = entry.author
        if template is None:
            template = self.settings['defaultLayout']
        return render_template(template, title=self.settings['title'],
                               content=content, date=date, tags=tags,
                               category=category, author=author)

    def _gettag(self, tag):
        """
        Render the Tags Page.
        :param tag: Tag for search
        :return: Rendered tags search.
        """
        postlist = self.blogrenderer.list_posts([tag])
        content = self.blogrenderer.generatetagpage(postlist)
        return render_template(self.settings['defaultLayout'],
                               title=self.settings['title'], content=content)

    def searchpages(self):
        """
        Render the search page. Must Be on Method POST
        :return: rendered search Page
        """
        postlist = self.blogrenderer.list_posts(search=request.form['search'])
        content = self.blogrenderer.generatetagpage(postlist)
        return render_template(self.settings['defaultLayout'],
                               title=self.settings['title'], content=content)

    def _getcategory(self, category):
        """
        Render a category searchpage
        :param category:
        :return: rendered category search page
        """
        postlist = self.blogrenderer.list_posts(category=category)
        content = self.blogrenderer.generatetagpage(postlist)
        return render_template(self.settings['defaultLayout'],
                               title=self.settings['title'], content=content)

    def _getauthor(self, author):
        """
        Render an author searchpage
        :param author: author parameter
        :return:  rendered author search page
        """
        postlist = self.blogrenderer.list_posts(author=author)
        content = self.blogrenderer.generatetagpage(postlist)
        return render_template(self.settings['defaultLayout'],
                               title=self.settings['title'], content=content)

    def run(self, **kwargs):
        self.app.run(**kwargs)
